{*****************************************************************************
 * UnitName:  CPKeyHook
 * Version:   1.3
 * Created:   27/09/2002 20:52:11
 * Purpose:   Global Keyboard Hook Unit and DLL.
 * Developer: BITLOGIC Software
 * Email:     vcl@bitlogic.co.uk
 * WebPage:   http://www.bitlogic.co.uk
 *****************************************************************************}

unit CPKeyHook;

interface

uses Windows, Messages, Classes, Forms;

const
  HOOKLIBNAME = 'keyhook.dll';
  WM_KEYHOOKMSG: DWORD = WM_USER+0;

function Hook_Start(WinHandle : HWND; MsgToSend : DWORD) : boolean;
  stdcall external HOOKLIBNAME name 'KeyboardHook_Start';

function Hook_Stop : boolean;
  stdcall external HOOKLIBNAME name 'KeyboardHook_Stop';

type
TKeyStates = packed record
  KeyDown : Boolean;
  ShiftDown: Boolean;
  AltDown: Boolean;
  CtrlDown: Boolean;
  ExtendedKey: Boolean;
  MenuKey: Boolean;
  KeyRepeated: Boolean;
  RepeatCount: integer;
  end;

TKeyNames = packed record
 KeyChar: Char;
 KeyExtName: array[0..100] of Char;
 end;

type
TKeyHookedEvent = procedure(Sender: TObject; AKeyStates: TKeyStates; AKeyNames: TKeyNames) of object;

type
  TCPKeyHook = class(TComponent)
  private
   FWindowHandle: HWND;
   FUserHookMsg: DWORD;
   FEnabled: Boolean;
   FKeyLayout: string;
   FOnKey: TKeyHookedEvent;
   procedure SetEnabled(AValue: boolean);
   procedure SetNoneStr(AValue: string);
   procedure SetUserHookMsg(AMsg: DWORD);
   procedure WndProc(var Msg: TMessage);
  protected
   procedure HookMsg(var msg : TMessage); //message WM_KEYHOOKMSG;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   function Start_KeyHook: boolean;
   function Stop_KeyHook: boolean;
  published
   property WindowHandle: HWND read FWindowHandle;
   property UserHookMsg: DWORD read FUserHookMsg write SetUserHookMsg;
   property Enabled: boolean read FEnabled write SetEnabled;
   property KeyboardLayout: string read FKeyLayout write SetNoneStr;
   property OnKey: TKeyHookedEvent read FOnKey write FOnKey;
  end;

var
 keyreps: integer;

implementation

procedure TCPKeyHook.SetNoneStr(AValue: string);begin;end;
procedure TCPKeyHook.SetEnabled(AValue: boolean);begin;end;

procedure TCPKeyHook.SetUserHookMsg(AMsg: DWORD);
begin
if AMsg = FUserHookMsg then exit;
FUserHookMsg := AMsg;
end;

constructor TCPKeyHook.Create(AOwner: TComponent);
var
KLayout : array[0..KL_NAMELENGTH] of char;
begin
 inherited Create(AOwner);
 {if not (csDesigning in ComponentState) then}
FWindowHandle := Classes.AllocateHWnd(WndProc);
FEnabled := False;
FUserHookMsg := WM_KEYHOOKMSG;
keyreps := 0;
GetKeyboardLayoutName(@KLayout);
FKeyLayout := KLayout;
end;

destructor TCPKeyHook.Destroy;
begin
TRY
Hook_Stop;
FINALLY
Classes.DeallocateHWnd(FWindowHandle);
inherited Destroy;
END;
end;

procedure TCPKeyHook.WndProc(var Msg: TMessage);
begin
if Msg.Msg = FUserHookMsg then
     try
     HookMsg(Msg);
     except
     Application.HandleException(Self);
     end
  else Msg.Result := DefWindowProc(FWindowHandle, Msg.Msg, Msg.WParam, Msg.LParam);
end;

procedure TCPKeyHook.HookMsg(var msg : TMessage);
var
FKeyState: TKeyStates;
FKeyNames: TKeyNames;
VKeyName : array[0..100] of char;
VKeyChar : char;
KBS : TKeyboardState;
begin
  fillchar(VKeyName,SizeOf(VKeyName),#0);
  FKeyNames.KeyChar := #0;
  FKeyState.KeyDown := (msg.lParam AND (1 shl 31)) = 0;
  FKeyState.KeyRepeated := (msg.lParam AND (1 shl 30)) <> 0;
  FKeyState.AltDown := (msg.lParam AND (1 shl 29)) <> 0;
  FKeyState.MenuKey := (msg.lParam AND (1 shl 28)) <> 0;
  FKeyState.ExtendedKey := (msg.lParam AND (1 shl 24)) <> 0;
  FKeyState.CtrlDown := (GetKeyState(VK_CONTROL) AND (1 shl 15)) <> 0;
  FKeyState.ShiftDown  := (GetKeyState(VK_SHIFT) AND (1 shl 15)) <> 0;
  if (FKeyState.KeyRepeated and FKeyState.KeyDown) then inc(keyreps)
  else keyreps := 0;
  FKeyState.RepeatCount := keyreps;
  GetKeyboardState(KBS);
  GetKeyNameText(msg.LParam,@VKeyName,SizeOf(VKeyName));
  move(VKeyName,FKeyNames.KeyExtName,SizeOf(VKeyName));
  //tasc := Char(ToAsciiEx(Msg.WParam,Msg.LParam, KBS, @nc, Msg.LParam,GetKeyboardLayout(0)));
  if ToAscii(Msg.WParam, Msg.LParam, KBS, @VKeyChar, 0) <> 0
  then FKeyNames.KeyChar := VKeyChar;
  if Assigned(FOnKey) then FOnKey(self,FKeyState,FKeyNames);
end;

function TCPKeyHook.Start_KeyHook: Boolean;
begin
Result := False;
if FEnabled then exit;
if Hook_Start(FWindowHandle,FUserHookMsg) then begin
   FEnabled := True;
   Result := True;
   end;
end;

function TCPKeyHook.Stop_KeyHook: Boolean;
begin
Result := False;
if Not FEnabled then exit;
if Hook_Stop then begin
   FEnabled := False;
   Result := True;
   end;
end;


end.
